﻿'///////////////////////////////////////////////////////////////////////
'
' UPDATED DEC 30, 2011 by Hazim Bitar to allow for Arduino/Servo communications.
' By: TechBitar/Hazim Bitar
'  Email: techbitar at gmail dot com
' This module provides sample code used to demonstrate the use
' of the KinectAudioSource for audio capture and beam tracking
'
' Copyright © Microsoft Corporation.  All rights reserved.  
' This code is licensed under the terms of the 
' Microsoft Kinect for Windows SDK (Beta) from Microsoft Research 
' License Agreement: http://research.microsoft.com/KinectSDK-ToU
'
'///////////////////////////////////////////////////////////////////////

'Arduino start
Imports System
Imports System.IO.Ports
Imports System.Threading
'Arduino end

Imports System.Text
Imports Microsoft.Kinect
Imports Coding4Fun.Kinect.Wpf

Namespace SkeletalTracking

	''' <summary>
	''' Interaction logic for MainWindow.xaml
	''' </summary>
	Partial Public Class MainWindow
		Inherits Window

        ' Arduino start
        Public Shared _continue As Boolean

        Public Shared ScreenMaxX As Integer = 500
        Public Shared ScreenMaxY As Integer = 390


        ' Define maximum thresholds for relating Kinect to Shoulder
        Public Shared sMaxKinectHandY As Single = 0.25
        Public Shared bMaxShoulder As Byte = 185

        Public Shared sMinKinectHandY As Single = 0.7
        Public Shared bMinShoulder As Byte = 47


        Public Shared rbPCR As RobobuilderLib.PCremote
        Public Shared rbWCK As RobobuilderLib.wckMotion

        Private commRb As SerialPort
        Private commStepper As SerialPort

        Public bRBConnected As Boolean = False
        Public bStepperConnected As Boolean = False

        Private sbKinectAngle As SByte

        Private bServoRange(15, 2) As Byte
        Private dDegreesRange(15, 2) As Double

        Private Const atMINwck = 0
        Private Const atMAXwck = 1

        Private Const X = 0
        Private Const Y = 1
        Private Const Z = 2


        Private WithEvents tmrStepper As System.Timers.Timer
        Private WithEvents sensor As KinectSensor

        Public Sub New()

            InitializeComponent()

        End Sub

        Private Sub Window_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)

            Me.txtRBComPort.Text = GetSetting("RobobuilderKinect", "RobobuilderSettings", "COMM Port", "1")
            Me.txtStepperCOMMPort.Text = GetSetting("RobobuilderKinect", "Stepper Settings", "COMM Port", "2")
            Me.txtStepperTimerInterval.Text = GetSetting("RobobuilderKinect", "Stepper Settings", "UpdateDelayMilisecs", "3000")


            ' will discover and initialize Kinect
            DiscoverKinectSensor()

            DisconnectFromRB()

            tmrStepper = New System.Timers.Timer()
            tmrStepper.Enabled = False

            ' define maximum ranges for servos
            bServoRange(14, atMINwck) = 125 : dDegreesRange(14, atMINwck) = -90 ' when at 125 this corresponds to -90 deg
            bServoRange(14, atMAXwck) = 207 : dDegreesRange(14, atMAXwck) = 0 ' and when at 207 this corresponds to 0

            bServoRange(13, atMINwck) = 0 : dDegreesRange(13, atMINwck) = 80
            bServoRange(13, atMAXwck) = 208 : dDegreesRange(13, atMAXwck) = -120

            bServoRange(15, atMINwck) = 140 : dDegreesRange(15, atMINwck) = 0
            bServoRange(15, atMAXwck) = 242 : dDegreesRange(15, atMAXwck) = 135


            bServoRange(11, atMINwck) = 48 : dDegreesRange(11, atMINwck) = 0
            bServoRange(11, atMAXwck) = 130 : dDegreesRange(11, atMAXwck) = -90

            bServoRange(10, atMINwck) = 48 : dDegreesRange(10, atMINwck) = 120
            bServoRange(10, atMAXwck) = 255 : dDegreesRange(10, atMAXwck) = -80

            bServoRange(12, atMINwck) = 13 : dDegreesRange(12, atMINwck) = 135
            bServoRange(12, atMAXwck) = 115 : dDegreesRange(12, atMAXwck) = 0

        End Sub


        Private Sub sensor_SkeletonFrameReady(sender As Object, e As Microsoft.Kinect.SkeletonFrameReadyEventArgs) Handles sensor.SkeletonFrameReady
            Dim skFrame As SkeletonFrame
            Dim skeletonSlot As Integer = 0
            Dim playerSkeleton As Skeleton

            skFrame = e.OpenSkeletonFrame

            If Not skFrame Is Nothing Then
                Dim skeletonData(skFrame.SkeletonArrayLength - 1) As Skeleton
                skFrame.CopySkeletonDataTo(skeletonData)
                playerSkeleton = (From s In skeletonData Where s.TrackingState = SkeletonTrackingState.Tracked Select s).FirstOrDefault()

                If Not playerSkeleton Is Nothing Then
                    SetEllipsePosition(headEllipse, playerSkeleton.Joints(JointType.Head), JointType.Head)
                    SetEllipsePosition(leftEllipse, playerSkeleton.Joints(JointType.HandLeft), JointType.HandLeft)
                    SetEllipsePosition(rightEllipse, playerSkeleton.Joints(JointType.HandRight), JointType.HandRight)

                    ' Kinect Angle
                    If playerSkeleton.Joints(JointType.Head).Position.Z = 0 Then
                        sbKinectAngle = 0
                    Else
                        sbKinectAngle = CSByte((180 * Math.Atan(playerSkeleton.Joints(JointType.Head).Position.X / playerSkeleton.Joints(JointType.Head).Position.Z)) / 3.14159)

                        If Math.Abs(sbKinectAngle) <= 5 Then ' set dead zone
                            sbKinectAngle = 0
                        Else
                            sbKinectAngle = CSByte(sbKinectAngle / 2) ' we want to take small steps; since this is an iteration this will help prevent overshoot
                        End If
                    End If
                    Me.txtAngleToKinect.Text = CStr(sbKinectAngle)


                    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''
                    ' LEFT ARM
                    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''
                    UpdateLeftArm(playerSkeleton)

                    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''
                    ' RIGHT ARM
                    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''
                    UpdateRightArm(playerSkeleton)

                End If

                Me.InvalidateVisual()
                'System.Windows.Forms.Application.DoEvents()

                skFrame.Dispose()
                skFrame = Nothing
            Else
                ' set Kinect angle to 127 will tell the Arduino we lost the person.
                ' In reality this will make the Arduino stop or enter an "environment scan mode" (the arduino decides hwat he wants to do with this info)
                sbKinectAngle = 127

            End If

        End Sub

        Private Sub UpdateRightArm(playerSkeleton As Microsoft.Kinect.Skeleton)
            ''''''''''''''''''''''''''''''''''''''''
            ' RIGHT ARM
            ''''''''''''''''''''''''''''''''''''''''

            ' Calculate Angles and values for Right Arm (Robobuilder's Left Arm).
            Dim dVectorRS_RElbowShoulder(2) As Double
            Dim dVectorRS_RWristElbow(2) As Double

            CalculateVector(dVectorRS_RElbowShoulder, playerSkeleton, JointType.ElbowRight, JointType.ShoulderRight)

            HEADKinectX.Text = CStr(dVectorRS_RElbowShoulder(X))
            HEADKinectY.Text = CStr(dVectorRS_RElbowShoulder(Y))
            HEADKinectZ.Text = CStr(dVectorRS_RElbowShoulder(Z))

            Dim dVectorRobobuilder(2) As Double
            dVectorRobobuilder(X) = 0 - dVectorRS_RElbowShoulder(Z)
            dVectorRobobuilder(Y) = 0 - dVectorRS_RElbowShoulder(Y)
            dVectorRobobuilder(Z) = 0 - dVectorRS_RElbowShoulder(X)

            Dim dRadius_LElbowShoulder As Double = CalculateVectorSize(dVectorRobobuilder)

            Dim dAzimuth_RElbowShoulder As Double = Math.Atan(dVectorRobobuilder(Y) / dVectorRobobuilder(X))
            Dim dElevation_RElbowShoulder As Double = Math.PI / 2 - Math.Acos(dVectorRobobuilder(Z) / dRadius_LElbowShoulder)
            If dVectorRobobuilder(X) < 0 Then  ' correction to prevent going fron +90 to -90 when the arm is upright
                If dVectorRobobuilder(Y) > 0 Then

                    dAzimuth_RElbowShoulder = Math.PI + dAzimuth_RElbowShoulder
                Else
                    dAzimuth_RElbowShoulder = -Math.PI + dAzimuth_RElbowShoulder
                End If

            Else
                ' don't touch azimuth
            End If

            ' Now calculate the Vector for the Elbow/Wrist
            CalculateVector(dVectorRS_RWristElbow, playerSkeleton, JointType.WristRight, JointType.ElbowRight)
            Dim dAngle_RWristElbow = Math.Acos((dVectorRS_RWristElbow(X) * dVectorRS_RElbowShoulder(X) + dVectorRS_RWristElbow(Y) * dVectorRS_RElbowShoulder(Y) + dVectorRS_RWristElbow(Z) * dVectorRS_RElbowShoulder(Z)) / (CalculateVectorSize(dVectorRS_RWristElbow) * CalculateVectorSize(dVectorRS_RElbowShoulder)))
            Me.HLKinectZ.Text = CStr(RadToDeg(dAngle_RWristElbow))


            ' Write Angles to Servos
            ' the trigonometric functions may return NaN when a skeleton is not in a realistic position.
            If Not Double.IsNaN(dAzimuth_RElbowShoulder) And _
                Not Double.IsNaN(dElevation_RElbowShoulder) And _
                Not Double.IsNaN(dAngle_RWristElbow) Then

                ' convert the angles to degress
                dAzimuth_RElbowShoulder = RadToDeg(dAzimuth_RElbowShoulder)
                dElevation_RElbowShoulder = RadToDeg(dElevation_RElbowShoulder)
                dAngle_RWristElbow = RadToDeg(dAngle_RWristElbow)

                HLKinectX.Text = CStr(dAzimuth_RElbowShoulder)
                HLKinectY.Text = CStr(dElevation_RElbowShoulder)

                Dim bServo10 As Byte = AngleToWckPosition(dAzimuth_RElbowShoulder, 10)
                Dim bServo11 As Byte = AngleToWckPosition(dElevation_RElbowShoulder, 11)
                Dim bServo12 As Byte = AngleToWckPosition(dAngle_RWristElbow, 12)

                Me.HRKinectX.Text = CStr(bServo10)
                Me.HRKinectY.Text = CStr(bServo11)
                Me.HRKinectZ.Text = CStr(bServo12)

                If bRBConnected Then
                    rbWCK.wckMovePos(10, bServo10, 2)
                    rbWCK.wckMovePos(11, bServo11, 2)
                    rbWCK.wckMovePos(12, bServo12, 2)
                End If
            End If
        End Sub


        Private Sub UpdateLeftArm(playerSkeleton As Microsoft.Kinect.Skeleton)
            ''''''''''''''''''''''''''''''''''''''''
            ' LEFT ARM
            ''''''''''''''''''''''''''''''''''''''''

            ' Calculate Angles and values for Left Arm (Robobuilder's Right Arm).
            Dim dVectorRS_LElbowShoulder(2) As Double
            Dim dVectorRS_LWristElbow(2) As Double

            CalculateVector(dVectorRS_LElbowShoulder, playerSkeleton, JointType.ElbowLeft, JointType.ShoulderLeft)

            'HEADKinectX.Text = CStr(dVectorRS_LElbowShoulder(X))
            'HEADKinectY.Text = CStr(dVectorRS_LElbowShoulder(Y))
            'HEADKinectZ.Text = CStr(dVectorRS_LElbowShoulder(Z))

            Dim dVectorRobobuilder(2) As Double
            dVectorRobobuilder(X) = 0 - dVectorRS_LElbowShoulder(Z)
            dVectorRobobuilder(Y) = dVectorRS_LElbowShoulder(Y)
            dVectorRobobuilder(Z) = dVectorRS_LElbowShoulder(X)

            Dim dRadius_LElbowShoulder As Double = CalculateVectorSize(dVectorRobobuilder)

            Dim dAzimuth_LElbowShoulder As Double = Math.Atan(dVectorRobobuilder(Y) / dVectorRobobuilder(X))
            Dim dElevation_LElbowShoulder As Double = Math.PI / 2 - Math.Acos(dVectorRobobuilder(Z) / dRadius_LElbowShoulder)
            If dVectorRobobuilder(X) < 0 Then  ' correction to prevent going fron +90 to -90 when the arm is upright
                If dVectorRobobuilder(Y) > 0 Then

                    dAzimuth_LElbowShoulder = Math.PI + dAzimuth_LElbowShoulder
                Else
                    dAzimuth_LElbowShoulder = -Math.PI + dAzimuth_LElbowShoulder
                End If

            Else
                ' don't touch azimuth
            End If

            ' Now calculate the Vector for the Elbow/Wrist
            CalculateVector(dVectorRS_LWristElbow, playerSkeleton, JointType.WristLeft, JointType.ElbowLeft)
            Dim dAngle_LWristElbow = Math.Acos((dVectorRS_LWristElbow(X) * dVectorRS_LElbowShoulder(X) + dVectorRS_LWristElbow(Y) * dVectorRS_LElbowShoulder(Y) + dVectorRS_LWristElbow(Z) * dVectorRS_LElbowShoulder(Z)) / (CalculateVectorSize(dVectorRS_LWristElbow) * CalculateVectorSize(dVectorRS_LElbowShoulder)))
            'Me.HLKinectZ.Text = CStr(RadToDeg(dAngle_LWristElbow))


            ' Write Angles to Servos
            ' the trigonometric functions may return NaN when a skeleton is not in a realistic position.
            If Not Double.IsNaN(dAzimuth_LElbowShoulder) And _
                Not Double.IsNaN(dElevation_LElbowShoulder) And _
                Not Double.IsNaN(dAngle_LWristElbow) Then

                ' convert the angles to degress
                dAzimuth_LElbowShoulder = RadToDeg(dAzimuth_LElbowShoulder)
                dElevation_LElbowShoulder = RadToDeg(dElevation_LElbowShoulder)
                dAngle_LWristElbow = RadToDeg(dAngle_LWristElbow)

                'HLKinectX.Text = CStr(dAzimuth_LElbowShoulder)
                'HLKinectY.Text = CStr(dElevation_LElbowShoulder)

                Dim bServo13 As Byte = AngleToWckPosition(dAzimuth_LElbowShoulder, 13)
                Dim bServo14 As Byte = AngleToWckPosition(dElevation_LElbowShoulder, 14)
                Dim bServo15 As Byte = AngleToWckPosition(dAngle_LWristElbow, 15)

                'Me.HRKinectX.Text = CStr(bServo13)
                'Me.HRKinectY.Text = CStr(bServo14)
                'Me.HRKinectZ.Text = CStr(bServo15)

                If bRBConnected Then
                    rbWCK.wckMovePos(13, bServo13, 2)
                    rbWCK.wckMovePos(14, bServo14, 2)
                    rbWCK.wckMovePos(15, bServo15, 2)
                End If
            End If
        End Sub


        Private Sub CalculateVector(ByRef vectorXYZ() As Double, playerSkeleton As Microsoft.Kinect.Skeleton, jEndJoint As Microsoft.Kinect.JointType, jStartJoint As Microsoft.Kinect.JointType)
            ' the vector is passed By Reference so we will write the results directly to the original argument passed.

            vectorXYZ(X) = playerSkeleton.Joints(jEndJoint).Position.X - playerSkeleton.Joints(jStartJoint).Position.X
            vectorXYZ(Y) = playerSkeleton.Joints(jEndJoint).Position.Y - playerSkeleton.Joints(jStartJoint).Position.Y
            vectorXYZ(Z) = playerSkeleton.Joints(jEndJoint).Position.Z - playerSkeleton.Joints(jStartJoint).Position.Z


        End Sub

        Private Function CalculateVectorSize(ByVal vector() As Double) As Double
            Return Math.Sqrt(vector(X) ^ 2 + vector(Y) ^ 2 + vector(Z) ^ 2)
        End Function

        Private Function RadToDeg(radians As Double) As Double
            Return radians * 180 / Math.PI

        End Function

        Private Function AngleToWckPosition(dAngle As Double, iServoNumber As Integer) As Byte

            Dim dAngleRange As Double = Math.Abs(dDegreesRange(iServoNumber, atMAXwck) - dDegreesRange(iServoNumber, atMINwck))
            Dim bWckRange As Byte = bServoRange(iServoNumber, atMAXwck) - bServoRange(iServoNumber, atMINwck)

            Dim bPos As Byte, dNormalizedAngle As Double, dClippedAngle As Double

            If dDegreesRange(iServoNumber, atMINwck) > dDegreesRange(iServoNumber, atMAXwck) Then
                ' in this case the servo and angle grow in an inverse relation
                ' clip actual angle to alowed limits
                dClippedAngle = Math.Min(dAngle, dDegreesRange(iServoNumber, atMINwck))
                dClippedAngle = Math.Max(dClippedAngle, dDegreesRange(iServoNumber, atMAXwck))

                ' normalize angle to range:
                dNormalizedAngle = dClippedAngle - dDegreesRange(iServoNumber, atMAXwck)

                bPos = CByte(Math.Floor(Math.Abs(CDbl(bWckRange) * (dNormalizedAngle / dAngleRange))))

                ' normalize position to range:
                bPos = bServoRange(iServoNumber, atMAXwck) - bPos

            Else
                ' clip actual angle to alowed limits
                dClippedAngle = Math.Min(dAngle, dDegreesRange(iServoNumber, atMAXwck))
                dClippedAngle = Math.Max(dClippedAngle, dDegreesRange(iServoNumber, atMINwck))

                ' normalize angle to range:
                dNormalizedAngle = dClippedAngle - dDegreesRange(iServoNumber, atMINwck)

                bPos = CByte(Math.Floor(Math.Abs(CDbl(bWckRange) * (dNormalizedAngle / dAngleRange))))

                ' normalize position to range:
                bPos = bPos + bServoRange(iServoNumber, atMINwck)

            End If
            Return bPos

        End Function

        Private Sub SetEllipsePosition(ByVal ellipse As FrameworkElement, ByVal kjoint As Joint, ByVal JID As JointType)

            Dim scaledJoint = kjoint.ScaleTo(ScreenMaxX, ScreenMaxY, 0.5F, 0.2F)

            Canvas.SetLeft(ellipse, scaledJoint.Position.X)
            Canvas.SetTop(ellipse, scaledJoint.Position.Y)

        End Sub

        Private Sub Window_Closed(ByVal sender As Object, ByVal e As EventArgs)

            'Cleanup
            sensor.Stop()
            sensor.Dispose()

            DisconnectFromRB()
            DisconnectFromStepper()

            ' Save configurations: COMM port numbers and delay for updating stepper
            SaveSetting("RobobuilderKinect", "RobobuilderSettings", "COMM Port", Me.txtRBComPort.Text)
            SaveSetting("RobobuilderKinect", "Stepper Settings", "COMM Port", Me.txtStepperCOMMPort.Text)
            SaveSetting("RobobuilderKinect", "Stepper Settings", "UpdateDelayMilisecs", Me.txtStepperTimerInterval.Text)


        End Sub

        Private Sub UpdateRobobuilderArms(ByVal bServo11 As Byte, ByVal bservo14 As Byte)
            ' left arm
            Dim bServo10 As Byte
            Dim bServo12 As Byte
            Dim dServo11 As Double = CDbl(bServo11)

            '               5E-05x3 - 0.0213x2 + 2.9618x - 46.011
            bServo10 = CByte(0.00005 * dServo11 ^ 3 - 0.0213 * dServo11 ^ 2 + 2.9618 * dServo11 - 46.011)

            '              8E-05x3 - 0.0339x2 + 4.7877x - 108.15
            bServo12 = CByte(0.00008 * (dServo11 ^ 3) - 0.0339 * (dServo11 ^ 2) + 4.7877 * dServo11 - 108.15)


            ' right arm
            Dim bServo13 As Byte
            Dim bServo15 As Byte
            Dim dServo14 As Double = CDbl(bservo14)

            '               5E-05x3 - 0.0176x2 + 2.0147x + 87.278
            bServo13 = CByte(0.00005 * (dServo14 ^ 3) - 0.0176 * (dServo14 ^ 2) + 2.0147 * dServo14 + 87.27)
            '               8E-05x3 - 0.0272x2 + 3.0598x + 24.756
            bServo15 = CByte(0.00008 * (dServo14 ^ 3) - 0.0272 * (dServo14 ^ 2) + 3.0598 * dServo14 + 24.756)

            SendArmPosition(bServo10, bServo11, bServo12, bServo13, bservo14, bServo15)

        End Sub

        Private Sub SendArmPosition(ByVal bServo10 As Byte, ByVal bServo11 As Byte, ByVal bServo12 As Byte, ByVal bServo13 As Byte, ByVal bServo14 As Byte, ByVal bServo15 As Byte)
            On Error GoTo ErrHandler

            If Not bRBConnected Then Exit Sub

            rbWCK.wckMovePos(10, bServo10, 1) ' 3rd parameter is Torque 0~4: 0=maximum
            rbWCK.wckMovePos(11, bServo11, 1)
            rbWCK.wckMovePos(12, bServo12, 1)
            rbWCK.wckMovePos(13, bServo13, 1)
            rbWCK.wckMovePos(14, bServo14, 1)
            rbWCK.wckMovePos(15, bServo15, 1)

            Me.InvalidateVisual()

            Exit Sub

ErrHandler:
            MsgBox("Unable to Update Servo Position" & vbCrLf & vbCrLf & Err.Description, vbCritical And vbOKOnly, "Error Connecting")
            Err.Clear()
        End Sub

        Private Sub ConnectToRB(ByVal sCOMMNum As String)
            On Error GoTo ErrHandler

            ' stop kinect sensor to avoid bombarding usb bus and ccreating a deadlock (ie hang).
            sensor.Stop()
            Thread.Sleep(500)

            commRb = New SerialPort("COM" & sCOMMNum, 115200, Parity.None, 8, StopBits.One)
            commRb.Open()

            rbPCR = New RobobuilderLib.PCremote(commRb)
            rbPCR.setDCmode(False)

            rbPCR.runMotion(7) ' basic posture
            rbPCR.setDCmode(True)

            rbWCK = New RobobuilderLib.wckMotion(rbPCR)

            Me.txtRBComStatus.Text = "Connected"
            bRBConnected = True

            Thread.Sleep(500)
            sensor.Start()

            Exit Sub

ErrHandler:
            MsgBox("Unable to Connect to Robobuilder" & vbCrLf & vbCrLf & Err.Description, vbCritical And vbOKOnly, "Error Connecting")
            rbPCR = Nothing
            If Not commRb Is Nothing Then
                If commRb.IsOpen Then commRb.Close()
            End If
            commRb = Nothing
            Err.Clear()

            Thread.Sleep(500)
            sensor.Start()

        End Sub

        Private Sub DisconnectFromRB()
            On Error GoTo ErrHandler

            bRBConnected = False
            If Not IsNothing(rbWCK) Then
                rbWCK.close()
                rbWCK = Nothing
            End If
            If Not IsNothing(rbPCR) Then
                rbPCR.Close()
                rbPCR = Nothing
            End If

            If Not IsNothing(commRb) Then
                If commRb.IsOpen Then commRb.Close()
            End If
            commRb = Nothing

            Me.txtRBComStatus.Text = "Disconnected"
            Exit Sub

ErrHandler:
            MsgBox("Unable to Disconnect from Robobuilder" & vbCrLf & vbCrLf & Err.Description, vbCritical And vbOKOnly, "Error Connecting")
            rbPCR = Nothing
            rbWCK = Nothing
            Err.Clear()

        End Sub

        Private Sub cmdConnectRB_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles cmdConnectRB.Click
            ConnectToRB(Me.txtRBComPort.Text)
        End Sub

        Private Sub cmdDisconnectRB_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles cmdDisconnectRB.Click
            DisconnectFromRB()
        End Sub


        Private Sub DiscoverKinectSensor()
            'Dim iterSensor As KinectSensor

            For Each iterSensor As KinectSensor In KinectSensor.KinectSensors
                If iterSensor.Status = KinectStatus.Connected Then
                    sensor = iterSensor
                    Exit For
                End If
            Next

            If sensor Is Nothing Then
                MsgBox("Could not find any valid Kinect Connected. Please restart the program", vbCritical)
                Exit Sub
            End If

            Select Case sensor.Status
                Case KinectStatus.Connected
                    ' kinect connected

                Case KinectStatus.Disconnected
                    MsgBox("Kinect is Disconnected", vbExclamation)


                Case KinectStatus.NotPowered
                    MsgBox("Kinect is Not Powered. Connect to Power", vbExclamation)

                Case Else
                    MsgBox("Unkown Kinect Status", vbExclamation)
            End Select

            If (sensor.Status = KinectStatus.Connected) Then
                InitializeKinect()
            End If
        End Sub

        Private Sub InitializeKinect()

            Dim parameters = New TransformSmoothParameters With {.Smoothing = 0.8F, .Correction = 0.3F, .Prediction = 0.4F, .JitterRadius = 1.0F, .MaxDeviationRadius = 0.5F}
            sensor.SkeletonStream.Enable(parameters)

            'add event to receive skeleton data
            'AddHandler sensor.SkeletonFrameReady, AddressOf nui_SkeletonFrameReady

            sensor.Start()
        End Sub

        Private Sub ConnectToStepper(ByVal sCOMMNum As String)
            On Error GoTo ErrHandler

            ' stop kinect sensor to avoid bombarding usb bus and ccreating a deadlock (ie hang).
            sensor.Stop()
            Thread.Sleep(500)

            commStepper = New SerialPort("COM" & sCOMMNum, 115200, Parity.None, 8, StopBits.One)
            commStepper.Open()

            Me.txtStepperCOMMStatus.Text = "Connected"
            bStepperConnected = True

            tmrStepper.Interval = CDbl(Me.txtStepperTimerInterval.Text)
            tmrStepper.Enabled = True

            Thread.Sleep(500)
            sensor.Start()

            Exit Sub

ErrHandler:
            MsgBox("Unable to Connect to Stepper " & vbCrLf & vbCrLf & Err.Description, vbCritical And vbOKOnly, "Error Connecting")
            tmrStepper.Enabled = False
            bStepperConnected = False
            commStepper.Close()
            Err.Clear()


            Thread.Sleep(500)
            sensor.Start()
        End Sub

        Private Sub DisconnectFromStepper()
            On Error GoTo ErrHandler

            bStepperConnected = False
            If Not IsNothing(commStepper) Then
                If commStepper.IsOpen Then commStepper.Close()
            End If
            commStepper = Nothing

            Me.txtStepperCOMMStatus.Text = "Disconnected"
            Exit Sub

ErrHandler:
            MsgBox("Unable to Disconnect from Stepper " & vbCrLf & vbCrLf & Err.Description, vbCritical And vbOKOnly, "Error Connecting")
            Err.Clear()

        End Sub

        Private Sub SendAngleToStepper()
            Dim bSend(3) As Byte
            Dim bpos As Byte

            If (sbKinectAngle < 0) Then
                bpos = CByte(255 - Math.Abs(CInt(sbKinectAngle)))
            Else
                bpos = CByte(sbKinectAngle)
            End If


            bSend(0) = 82
            bSend(1) = bpos
            bSend(2) = 13

            If bStepperConnected Then
                commStepper.Write(bSend, 0, 3)
            End If
        End Sub

        Private Sub tmrStepper_Elapsed(sender As Object, e As System.Timers.ElapsedEventArgs) Handles tmrStepper.Elapsed
            Debug.Print("Timer fired: Angle being sent " & CStr(sbKinectAngle) & vbCrLf)

            SendAngleToStepper()
        End Sub

        Private Sub cmdConnectStepper_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles cmdConnectStepper.Click
            ConnectToStepper(Me.txtStepperCOMMPort.Text)
        End Sub

        Private Sub cmdDisconnectStepper_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles cmdDisconnectStepper.Click
            DisconnectFromStepper()
        End Sub

        Protected Overrides Sub Finalize()
            MyBase.Finalize()
        End Sub
    End Class

End Namespace
'Private Sub SendToArduino(ByVal joint As Joint, ByVal JID As JointID)

'    Dim scaledJoint = joint.ScaleTo(ScreenMaxX, ScreenMaxY, 0.5F, 0.2F)          

'    If JID = JointID.HandRight Then

'        HRKinectX.Text = scaledJoint.Position.X & " / " & CStr(joint.Position.X)
'        HRKinectY.Text = scaledJoint.Position.Y & " / " & CStr(joint.Position.Y)

'    End If

'    If JID = JointID.HandLeft Then
'        HLKinectX.Text = scaledJoint.Position.X & " / " & CStr(joint.Position.X)
'        HLKinectY.Text = scaledJoint.Position.Y & " / " & CStr(joint.Position.Y)
'    End If

'    ' Update Robobuilder Position
'    UpdateRobobuilderArms(scaledJoint.Position.X, scaledJoint.Position.Y, 1, JID)

'    ' Arduino start
'    ArduinoSendByte(scaledJoint.Position.X, scaledJoint.Position.Y, 1, JID)
'    ' Arduino end

'End Sub


'Private Sub ArduinoSendByte(ByVal kinect_x As Single, ByVal kinect_y As Single, ByVal kinect_z As Single, ByVal kinect_j As Integer)
'    Dim x, y, z, j As Byte
'    Dim sx, sy As Single
'    Dim HowOften As Integer
'    ComStatus.Text = "NA"

'    x = 0 'Math.Abs(CByte(kinect_x))
'    y = 0 'Math.Abs(CByte(kinect_y))
'    z = 0 'CByte(kinect_z)
'    j = 0 'CByte(kinect_j)
'    x = x
'    Dim ArduinoBuffer() As Byte = {x, y, z, j}
'    If _serialPort.IsOpen Then
'        ComStatus.Text = "OK"
'        _serialPort.Write(ArduinoBuffer, 0, ArduinoBuffer.Length)
'    End If
'End Sub


'Private Sub ArduinoOpenSerial()
'    If Not _serialPort.IsOpen Then
'        _serialPort.Open()
'    Else
'        MsgBox("ARDUINO: SERIAL PORT CANNOT BE OPENED")
'    End If
'    _continue = True
'End Sub

'Private Sub ArduinoCloseSerial()
'    If _serialPort.IsOpen Then
'        _serialPort.Close()
'    End If
'End Sub

'Private Sub ArduinoSetSerial()
'    Dim ArduinoCom As String = txtRBComPort.Text
'    _serialPort = New SerialPort()
'    _serialPort.PortName = "COM" + Trim(txtRBComPort.Text)
'    _serialPort.BaudRate = 9600
'    ' _serialPort.Parity = 0
'    _serialPort.DataBits = 8
'    '  _serialPort.StopBits = 1
'    _serialPort.Handshake = 0
'    _serialPort.ReadTimeout = 500
'    _serialPort.WriteTimeout = 500

'End Sub